"""
Utilities for working with set partitions.
"""

def canonical_partition(l, include_sign=False):
    """
    Given a list of partition labels l, returns the list corresponding to the
    same partition, but with labels occurring in sorted order. If
    `include_sign` is True, also returns the sign of the permutation achieving
    the sorting.
    """
    d = len(set(l))
    all_labels = range(d)

    labels_in_order = []
    counter = 0
    sign = 1
    canonical_label_dict = dict()
    ret = []

    for x in l:
        label = canonical_label_dict.get(x)

        if label is None:
            for l2 in labels_in_order:
                if l2 > x:
                    sign *= -1
            labels_in_order.append(x)

            label = counter
            canonical_label_dict[x] = counter
            counter += 1

        ret.append(label)

    if include_sign:
        return (tuple(ret), sign)
    else:
        return tuple(ret)
