"""
Merges output of script_check_positivity.py if the `outer_slice` argument is
used there to get partial results (e.g. if parallelizing the computation).

Usage: sage -python script_merge_check_positivity_results *.out
"""

import argparse
import glob

from sage.all import *


PARSER = argparse.ArgumentParser(
    description=('Merges partial output of script_check_positivity.py if ' +
                 'the `outer_slice` argument is used there.'))
PARSER.add_argument(
    'filenames',
    nargs='*',
    type=str,
    help=('log files of the output of script_check_positivity.py for ' +
          'different slices to be merged'))
PARSER.add_argument(
    '--d',
    type=int,
    default=None,
    help=('underlying dimension of the calculation ($d = 6$ for our main ' +
          'result)'))
ARGS = PARSER.parse_args()


def main():
    M_values = []
    Id_values = []
    final_values = []

    for filename in ARGS.filenames:
        with open(filename, 'r') as f:
            this_M_value, this_Id_value, this_final_value = map(
                lambda s: sage_eval(s.strip()),
                f.readlines()[-3:])
            M_values.append(this_M_value)
            Id_values.append(this_Id_value)
            final_values.append(this_final_value)

    M_value = sum(M_values)
    Id_value = sum(Id_values)
    final_value = sum(final_values)

    if ARGS.d is not None:
        final_value_check = M_value + 1 / Integer(ARGS.d) * Id_value
        assert final_value == final_value_check

    if Id_value > 0 and final_value >= 0:
        print 'Success!'

    if Id_value > 0:
        print 'v^T I v = |v|^2 = %s > 0' % str(Id_value)
    else:
        print 'Error: basis vector projected to zero'

    if final_value >= 0:
        print 'v^T (M + I/d) v = %s >= 0' % str(final_value)
    else:
        print 'Error: v^T (M + I/d) v = %s < 0' % str(final_value)


if __name__ == '__main__':
    main()
