"""Young tableaux utilities."""

import collections as co

from sage.all import *


def default_standard_tableau(l):
    """Returns the tableau of shape l filled increasing across rows."""
    ret = []

    c = 1
    for i in l:
        if i > 0:
            ret.append(range(c, c + i))
            c = c + i

    return StandardTableau(ret)


def ssyt_row_stabilizer(T):
    """
    For a semistandard Young tableau T, returns the subgroup of the row
    stabliizer of the default standard tableau of the same shape as T that
    fixes the labels of each row of T.
    """
    permutables = []

    for row, row_ixs in zip(T, default_standard_tableau(T.shape())):
        row_dict = co.defaultdict(list)
        for i, i_ix in zip(row, row_ixs):
            row_dict[i].append(i_ix)
        permutables.extend(row_dict.values())

    gens = []
    for l in permutables:
        if len(l) == 1:
            continue

        gens.extend(zip(l[:-1], l[1:]))

    return PermutationGroup(gens=gens, domain=range(1, len(T.entries()) + 1))
